
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layApplication.h"
#include "layExceptions.h"
#include "tlInternational.h"
#include "tlException.h"
#include "tlLog.h"

#include <iostream>

#ifdef _MSC_VER // for VC++ 

//  for VC++ provide a wrapper for main.
#include <Windows.h>

int main (int argc, char *argv []);

int WINAPI 
wWinMain(HINSTANCE hInstance, HINSTANCE prevInstance, LPWSTR lpCmdLine, int nShowCmd)
{
  int argCount = 0;
  LPWSTR *szArgList = CommandLineToArgvW(GetCommandLine(), &argCount);

  //  fail safe behaviour
  if (!szArgList) {
	  MessageBox(NULL, L"Unable to parse command line", L"Error", MB_OK);
	  return 10;
  }

  char **argv = new char *[argCount];
  for (int i = 0; i < argCount; i++) {
	  QString a;
	  for (WCHAR *wc = szArgList [i]; *wc; ++wc) {
		  a += QChar (*wc);
	  }
	  QByteArray aa = a.toAscii ();
	  argv [i] = new char [aa.size () + 1];
	  strcpy (argv [i], aa.constData ());
  }

  int ret = main (argCount, argv);

  for (int i = 0; i < argCount; i++) {
	  delete[] argv [i];
  }
  delete[] argv;

  LocalFree(szArgList);
  return ret;
}
#endif

int 
main (int argc, char *argv [])
{
#if defined(Q_WS_X11) || defined(Q_WS_MAC)
  //  KLUDGE: the following statement overrides the LANG locale provided by the system.
  //  - The reason is, that Qt employs a function called _XInitOM that seems not to be 
  //    capable of handling "UTF8" LANG types. This makes QTextEdit widgets with HTML 
  //    content SIGSEGV when text is marked.
  //  - On MAC OX and when e german locale is set, floating point numbers like "0.001"
  //    cannot be read, since in a german locale, the decimal separator is ",".
  putenv ((char *) "LANG=C");
#endif

  int result = 0;

  try {

    lay::Application app (argc, argv);

    if (app.has_gui ()) {

      BEGIN_PROTECTED_CLEANUP

      result = app.run ();

      END_PROTECTED_CLEANUP {
        result = 1;
      }

    } else {
      result = app.run ();
    }

  } catch (std::exception &ex) {
    tl::error << ex.what ();
    result = 1;
  } catch (tl::Exception &ex) {
    tl::error << ex.msg ();
    result = 1;
  } catch (...) {
    tl::error << tl::translate ("unspecific error");
    result = 1;
  }

  return result;

}


